/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class COperation extends CCalculBase {
	public static final String parouv ="(";
	public static final String parfer = ")";
	public static final String zero = "0";
	public static final String zeroComp = "0+0*i";	
	
  CCalculBase operande1;
  CCalculBase operande2;
  int operateur;
  // Attributs pour optimisation
  Complexe c1 = new Complexe();
  Complexe c2 = new Complexe();

  /** Creates new COperation */
  public COperation() {
  }
  public COperation(CListeObjets liste, CCalculBase operande1Init, 
      CCalculBase operande2Init, int operateurInit) {
    super(liste);
    operande1 = operande1Init;
    operande2 = operande2Init;
    operateur = operateurInit;
  }
 
  @Override
  protected long nature() {
  	return natOperation;
  }  

  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase cloneOperande1 = operande1.getClone(listeSource, listeCible); 
    CCalculBase cloneOperande2 = operande2.getClone(listeSource, listeCible); 
    CCalculBase ptcalc = new COperation(listeCible, cloneOperande1, cloneOperande2, operateur);
    return ptcalc;
  }     
  
  @Override
  public boolean dependDe(CElementBase p) {
  	return operande1.dependDe(p) || operande2.dependDe(p);
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	return operande1.dependDePourReclassement(p) || operande2.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return operande1.dependDePourBoucle(p) || operande2.dependDePourBoucle(p);
  }

  @Override
	public boolean existe(boolean infoRandom) {
  	return operande1.existe(infoRandom) && operande2.existe(infoRandom);
  }

  private double resultatBase(double op1, double op2) throws ErreurCalculException {
    double denom;
    switch (operateur)
    {
      case Operateur.Plus:
        return op1 + op2;
      case Operateur.Moin:
        return op1 - op2;
      case Operateur.Mult: 
        return op1 * op2;
      //  revoir pour gestion des exceptions
      case Operateur.Divi:
         denom = op2;
         if (denom == 0)
         {
           throw new ErreurCalculException();
         }
         else return op1 / denom;
      case Operateur.Inf:
        if (op1 < op2)
                return 1;
        else return 0;
      case Operateur.Sup:
        if (op1 > op2)
                return 1;
        else return 0;
      case Operateur.InfOuEgal:
        if (op1 <= op2)
                return 1;
        else return 0;
      case Operateur.SupOuEgal:
        if (op1 >= op2)
                return 1;
        else return 0;
      case Operateur.Egalite:
        if (op1 == op2)
                return 1;
        else return 0;
      case Operateur.Diff:
        if (op1 != op2)
                return 1;
        else return 0;
      default: return 0;
    } // switch Operateur
  }

  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
     double o1 = operande1.resultat(infoRandom);
     double o2 = operande2.resultat(infoRandom);
     return resultatBase(o1, o2);
  }

  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
     double o1 = operande1.resultatFonction(infoRandom, valeurParametre);
     double o2 = operande2.resultatFonction(infoRandom, valeurParametre);
     return resultatBase(o1, o2);
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
     double o1 = operande1.resultatFonction(infoRandom, valeurParametre);
     double o2 = operande2.resultatFonction(infoRandom, valeurParametre);
     return resultatBase(o1, o2);
  }
  
  // La fonction suivante est utilise par les deux fonctions suivantes
  private void resultatComplexeBase(boolean infoRandom, Complexe op1, Complexe op2, Complexe zRes)
                              throws ErreurCalculException {
    switch (operateur)
    {
      case Operateur.Plus:
              zRes.x = op1.x + op2.x;
              zRes.y = op1.y + op2.y;
              return;
       case Operateur.Moin:
              zRes.x = op1.x - op2.x;
              zRes.y = op1.y - op2.y;
              return;
      case Operateur.Mult: 
              zRes.x = op1.x*op2.x - op1.y*op2.y;
              zRes.y = op1.x*op2.y + op1.y*op2.x;
              return;
      case Operateur.Divi:
        {
          double mod2 = op2.x*op2.x + op2.y*op2.y;
          if (mod2 == 0)
          {
            throw new ErreurCalculException();
          }
          else
          {
            zRes.x = (op1.x*op2.x + op1.y*op2.y)/mod2;
            zRes.y = (op1.y*op2.x - op1.x*op2.y)/mod2;
            return;
          }
        }
      case Operateur.Inf:
        // Si on est dans C, on n'accepte de comparer deux complexes
        // que s'ils sont rels
        {
          zRes.x = 0;
          zRes.y = 0;
          if ((op1.y != 0) || (op2.y != 0))
          {
            throw new ErreurCalculException();
          }
          else
          {
            double mod1 = op1.x*op1.x + op1.y*op1.y;
            double mod2 = op2.x*op2.x + op2.y*op2.y;
            if (mod1 < mod2) zRes.x = 1;
            else zRes.x = 0;
            return;
          }
        }
      case Operateur.Sup:
        // Si on est dans C, on n'accepte de comparer deux complexes
        // que s'ils sont rels
        {
          zRes.x = 0;
          zRes.y = 0;
          if ((op1.y != 0) || (op2.y != 0))
          {
            throw new ErreurCalculException();
          }
          else
          {
            if (op1.x > op2.x) zRes.x = 1;
            else zRes.x = 0;
            return;
          }
        }
      case Operateur.InfOuEgal:
        // Si on est dans C, on n'accepte de comparer deux complexes
        // que s'ils sont rels
        {
          zRes.x = 0;
          zRes.y = 0;
          if ((op1.y != 0) || (op2.y != 0))
          {
            throw new ErreurCalculException();
          }
          else
          {
            if (op1.x <= op2.x) zRes.x = 1;
            else zRes.x = 0;
            return;
          }
        }
      case Operateur.SupOuEgal:
        // Si on est dans C, on n'accepte de comparer deux complexes
        // que s'ils sont rels
        {
          zRes.x = 0;
          zRes.y = 0;
          if ((op1.y != 0) || (op2.y != 0))
          {
            throw new ErreurCalculException();
          }
          else
          {
            if (op1.x >= op2.x) zRes.x = 1;
            else zRes.x = 0;
            return;
          }
        }
      case Operateur.Egalite:
        zRes.y = 0;
        if ((op1.x == op2.x) && (op1.y == op2.y))
                zRes.x = 1;
        else zRes.x = 0;
        return;
      case Operateur.Diff:
        zRes.y = 0;
        if ((op1.x != op2.x) || (op1.y != op2.y))
                zRes.x = 1;
        else zRes.x = 0;
        return;
    } // switch Operateur   
  }

  // Ajout pour la version 2.0 calculs sur les complexes.
  // Le rsultat est renvoy dans z
  @Override
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
    operande1.resultatComplexe(infoRandom, c1);
    operande2.resultatComplexe(infoRandom, c2);
    resultatComplexeBase(infoRandom, c1, c2, zRes);
  }
  
  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
    operande1.resultatFonctionComplexe(infoRandom, valeurParametre, c1);
    operande2.resultatFonctionComplexe(infoRandom, valeurParametre, c2);
    resultatComplexeBase(infoRandom, c1, c2, zRes);
  }

  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
    operande1.resultatFonctionComplexe(infoRandom, valeurParametre, c1);
    operande2.resultatFonctionComplexe(infoRandom, valeurParametre, c2);
    resultatComplexeBase(infoRandom, c1, c2, zRes);
  }
  
  @Override
  public boolean  dependDeVariable(int ind) {
    return operande1.dependDeVariable(ind) || operande2.dependDeVariable(ind);
  }
  
  @Override
  public CCalculBase getCopie() {
   	return new COperation(listeProprietaire, operande1.getCopie(), operande2.getCopie(),
      operateur);
  }
  
  // A t remani pour la version 3.8 pour viter des parenthses inutiles pour les tests dont
  // la priorit est infrieure  tous les autres oprateurs
  @Override
	public String chaineCalcul(String[] varFor) {
  	String ch1 ="", ch2 = "", ch = "";
  	switch (operateur)
  	{
  		case Operateur.Plus:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else {
  				// else ch1 = operande1.chaineCalcul(varFor); 
  				// Rajout version 4.6
  				if (operande1.nature() == natMoinsUnaire) ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Plus) || (op2 == Operateur.Moin) || (op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = parouv + ch1 + "+" + ch2 + parfer;
  			break;
  		case Operateur.Moin:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor);
  			}
  			else {
  				// ch1 = operande1.chaineCalcul(varFor); 
					// Rajout version 4.6
  				if (operande1.nature() == natMoinsUnaire) ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor);
  			}
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = parouv + ch1 + "-" + ch2 + parfer;
  			break;
  		case Operateur.Mult: 
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			// Modification version 4.5.2 : Si une multiplication est suivie d'un quotient on garde les parenthses pour le quotient
  			/*
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			*/
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if (op2 == Operateur.Mult)
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = parouv + ch1 + "*" + ch2 + parfer;
  			break;
  		case Operateur.Divi:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			ch2 = operande2.chaineCalcul(varFor); 
  			ch = parouv + ch1 + "/" + ch2 + parfer;
  			break;
  		case Operateur.Inf:
  		case Operateur.Sup:
  		case Operateur.InfOuEgal:
  		case Operateur.SupOuEgal:
  		case Operateur.Egalite:
  		case Operateur.Diff:
  			// Modifi version 4.6 pour viter des parenthses inutiles.
  			// Tout a priorit sur les tests
  			/*
  			if (operande1.getClass() == COperation.class) {
  				ch1 = operande1.chaineCalculSansPar(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if (Operateur.estTest(op2))
  					ch2 = operande2.chaineCalcul(varFor);
  				else ch2 = operande2.chaineCalculSansPar(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor);
  			*/
  			if (operande1.nature() == natOperation) {
  				COperation oper1 = (COperation) operande1;
  				if (Operateur.estTest(oper1.operateur)) ch1 = operande1.chaineCalcul(varFor);
  				else ch1 = operande1.chaineCalculSansPar(varFor);
  			}
  			else ch1 = operande1.chaineCalculSansPar(varFor);
  			if (operande2.nature() == natOperation) {
  				COperation oper2 = (COperation) operande2;
  				if (Operateur.estTest(oper2.operateur)) ch2 = operande2.chaineCalcul(varFor);
  				else ch2 = operande2.chaineCalculSansPar(varFor);
  			}
  			else ch2 = operande2.chaineCalculSansPar(varFor);
 			ch = parouv + ch1 + Operateur.chaineOperateur(operateur) + ch2 + parfer;
  	} // switch Operateur}
  	// Au cas o le calcul commenait par un moins unaire, la chaine commence par "0-"
  	if (ch.substring(0, 3).equals("(0-")) ch = parouv + "-" + ch.substring(3);
  	return ch;
  }
  
  // A t remani pour la version 3.8 pour viter des parenthses inutiles pour les tests dont
  // la priorit est infrieure  tous les autres oprateurs  
  @Override
	public String chaineCalculSansPar(String[] varFor) {
  	String ch1 = "", ch2 = "", ch = "";
  	switch (operateur) {
  		case Operateur.Plus:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else {
  				// ch1 = operande1.chaineCalcul(varFor); 
  				// Rajout version 4.6
  				if (operande1.nature() == natMoinsUnaire) ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor);
  			}
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Plus) || (op2 == Operateur.Moin) || (op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = ch1 + "+" + ch2;
  			break;
  		case Operateur.Moin:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor);
  			}
  			else {
  				// ch1 = operande1.chaineCalcul(varFor); 
					// Rajout version 4.6
  				if (operande1.nature() == natMoinsUnaire) ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor);
  			}
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = ch1 + "-" + ch2;
  			break;
  		case Operateur.Mult: 
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			// Modification version 4.5.2 : Pour ne pas perturber l'utilisateur on garde sa formule telle quelle
  			// en cas de produit par un quotient
  			/*
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			*/
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if (op2 == Operateur.Mult)
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = ch1 + "*" + ch2;
  			break;
  		case Operateur.Divi:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			ch2 = operande2.chaineCalcul(varFor); 
  			ch = ch1 + "/" + ch2;
  			break;
  		// Modifi version 3.0
  		case Operateur.Inf:
  		case Operateur.Sup:
  		case Operateur.InfOuEgal:
  		case Operateur.SupOuEgal:
  		case Operateur.Egalite:
  		case Operateur.Diff:
  			// Modifi version 4.6 pour viter des parenthses inutiles.
  			/*
  			if (operande1.getClass() == COperation.class) {
  				ch1 = operande1.chaineCalculSansPar(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if (Operateur.estTest(op2))
  					ch2 = operande2.chaineCalcul(varFor);
  				else ch2 = operande2.chaineCalculSansPar(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor);
  			*/
  			if (operande1.nature() == natOperation) {
  				COperation oper1 = (COperation) operande1;
  				if (Operateur.estTest(oper1.operateur)) ch1 = operande1.chaineCalcul(varFor);
  				else ch1 = operande1.chaineCalculSansPar(varFor);
  			}
  			else ch1 = operande1.chaineCalculSansPar(varFor);
  			if (operande2.nature() == natOperation) {
  				COperation oper2 = (COperation) operande2;
  				if (Operateur.estTest(oper2.operateur)) ch2 = operande2.chaineCalcul(varFor);
  				else ch2 = operande2.chaineCalculSansPar(varFor);
  			}
  			else ch2 = operande2.chaineCalculSansPar(varFor);
  			ch = ch1 + Operateur.chaineOperateur(operateur) + ch2;
  	} // switch Operateur}
  	// Pour un calcul complexe nul la chane renvoye est "0+0*i" on renvoie alors "0"
  	if (ch.equals(zeroComp)) return zero;
  	// Au cas o le calcul commenait par un moins unaire, la chaine commence par "0-"
  	if (ch.substring(0, 2).equals("0-")) ch = ch.substring(1);
  	
  	return ch;		
	}
  
  @Override
	public String chaineLatex(String[] varFor) {
  	String ch1 ="", ch2 = "", ch = "";
  	switch (operateur)
  	{
  		case Operateur.Plus:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);
  			}
  			else {
  				// ch1 = operande1.chaineLatex(varFor); 
					// Rajout version 4.6
  				if (operande1.nature() == natMoinsUnaire) ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);
  			}
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Plus) || (op2 == Operateur.Moin) || (op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineLatexSansPar(varFor);
  				else ch2 = operande2.chaineLatex(varFor); 
  			}
  			else ch2 = operande2.chaineLatex(varFor); 
  			ch = parouvLatex + ch1 + "+" + ch2 + parferLatex;
  			break;
  		case Operateur.Moin:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);
  			}
  			else {
  				// ch1 = operande1.chaineLatex(varFor); 
  				if (operande1.nature() == natMoinsUnaire) ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);  					
  			}
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineLatexSansPar(varFor);
  				else ch2 = operande2.chaineLatex(varFor); 
  			}
  			else ch2 = operande2.chaineLatex(varFor); 
  			ch = parouvLatex + ch1 + "-" + ch2 + parferLatex;
  			break;
  		case Operateur.Mult: 
  			boolean bsignemult = false;
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor); 
  				if (op1 == Operateur.Divi) bsignemult = true;
  			}
  			else ch1 = operande1.chaineLatex(varFor);
  			/*
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineLatexSansPar(varFor);
  				else ch2 = operande2.chaineLatex(varFor); 
  			}
  			*/
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if (op2 == Operateur.Mult)
  					ch2 = operande2.chaineLatexSansPar(varFor);
  				else ch2 = operande2.chaineLatex(varFor);
  				if (op2 == Operateur.Divi) bsignemult = true;
  			}
  			else ch2 = operande2.chaineLatex(varFor); 
  			if (bsignemult || FonteNom.chiffre(ch2.charAt(0))) ch = parouvLatex + ch1 + "\\times " + ch2 + parferLatex; // Espace ncessaire
  			else ch = parouvLatex + ch1 + ch2 + parferLatex;
  			break;
  		case Operateur.Divi:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor); 
  			}
  			else ch1 = operande1.chaineLatex(varFor); 
  			ch2 = operande2.chaineLatex(varFor); 
  			ch = "\\frac{" + ch1 + "} {" + ch2 + "}";
  			break;
  		case Operateur.Inf:
  		case Operateur.Sup:
  		case Operateur.InfOuEgal:
  		case Operateur.SupOuEgal:
  		case Operateur.Egalite:
  		case Operateur.Diff:
  			// Modifi version 4.6
  			/*
  			if (operande1.getClass() == COperation.class) {
  				ch1 = operande1.chaineLatexSansPar(varFor); 
  			}
  			else ch1 = operande1.chaineLatex(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if (Operateur.estTest(op2))
  					ch2 = operande2.chaineLatex(varFor);
  				else ch2 = operande2.chaineLatexSansPar(varFor); 
  			}
  			else ch2 = operande2.chaineLatex(varFor);
  			*/
  			if (operande1.nature() == natOperation) {
  				COperation oper1 = (COperation) operande1;
  				if (Operateur.estTest(oper1.operateur)) ch1 = operande1.chaineLatex(varFor);
  				else ch1 = operande1.chaineLatexSansPar(varFor);
  			}
  			// else ch1 = operande1.chaineCalculSansPar(varFor);  // Correction dfaut version 4.7.2
  			else ch1 = operande1.chaineLatexSansPar(varFor);
  			if (operande2.nature() == natOperation) {
  				COperation oper2 = (COperation) operande2;
  				if (Operateur.estTest(oper2.operateur)) ch2 = operande2.chaineLatex(varFor);
  				else ch2 = operande2.chaineLatexSansPar(varFor);
  			}
  			else ch2 = operande2.chaineLatexSansPar(varFor);
  			
  			ch = parouvLatex + ch1 + Operateur.chaineOperateurLatex(operateur) + ch2 + parferLatex;
  	} // switch Operateur}
  	// Au cas o le calcul commenait par un moins unaire, la chaine commence par "0-"
  	if (ch.substring(0, 3).equals("(0-")) ch = parouv + "-" + ch.substring(3);
  	return ch;
  }
  
  @Override
	public String chaineLatexSansPar(String[] varFor) {
  	String ch1 = "", ch2 = "", ch = "";
  	switch (operateur) {
  		case Operateur.Plus:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);
  			}
  			else {
  				// ch1 = operande1.chaineLatex(varFor);
					// Rajout version 4.6
  				if (operande1.nature() == natMoinsUnaire) ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);  					  				
  			}
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Plus) || (op2 == Operateur.Moin) || (op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineLatexSansPar(varFor);
  				else ch2 = operande2.chaineLatex(varFor); 
  			}
  			else ch2 = operande2.chaineLatex(varFor); 
  			ch = ch1 + "+" + ch2;
  			break;
  		case Operateur.Moin:
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);
  			}
  			else {
  				// ch1 = operande1.chaineLatex(varFor); 
					// Rajout version 4.6
  				if (operande1.nature() == natMoinsUnaire) ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);  					
  			}
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineLatexSansPar(varFor);
  				else ch2 = operande2.chaineLatex(varFor); 
  			}
  			else ch2 = operande2.chaineLatex(varFor); 
  			ch = ch1 + "-" + ch2;
  			break;
  		case Operateur.Mult:
  			boolean bsignemult = false;
  			if (operande1.nature() == natOperation) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineLatexSansPar(varFor);
  				else ch1 = operande1.chaineLatex(varFor);
  				if (op1 == Operateur.Divi) bsignemult = true;
  			}
  			else ch1 = operande1.chaineLatex(varFor);
  			/*
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineLatexSansPar(varFor);
  				else ch2 = operande2.chaineLatex(varFor); 
  			}
  			*/
  			if (operande2.nature() == natOperation) {
  				int op2 = ((COperation) operande2).operateur;
  				if (op2 == Operateur.Mult)
  					ch2 = operande2.chaineLatexSansPar(varFor);
  				else ch2 = operande2.chaineLatex(varFor); 
  				if (op2 == Operateur.Divi) bsignemult = true;
  			}
  			else ch2 = operande2.chaineLatex(varFor); 
  			// ch = ch1 + "*" + ch2; // Scial pour Latex, pas de signe multiplicatif
  			if (bsignemult || FonteNom.chiffre(ch2.charAt(0))) ch = ch1 + "\\times " + ch2; // Espace de fin ncessaire
  			else ch = ch1 + ch2;
  			break;
  		case Operateur.Divi:
  			ch1 = operande1.chaineLatexSansPar(varFor); 
  			ch2 = operande2.chaineLatexSansPar(varFor); 
  			ch = "\\frac{" + ch1 +"}{" + ch2 + "}";
  			break;
  		// Modifi version 3.0
  		case Operateur.Inf:
  		case Operateur.Sup:
  		case Operateur.InfOuEgal:
  		case Operateur.SupOuEgal:
  		case Operateur.Egalite:
  		case Operateur.Diff:
  			// Modifi version 4.6
  			/*
  			if (operande1.getClass() == COperation.class) {
  				ch1 = operande1.chaineLatexSansPar(varFor); 
  			}
  			else ch1 = operande1.chaineLatex(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if (Operateur.estTest(op2))
  					ch2 = operande2.chaineLatex(varFor);
  				else ch2 = operande2.chaineLatexSansPar(varFor); 
  			}
  			else ch2 = operande2.chaineLatex(varFor);
  			*/
  			if (operande1.nature() == natOperation) {
  				COperation oper1 = (COperation) operande1;
  				if (Operateur.estTest(oper1.operateur)) ch1 = operande1.chaineLatex(varFor);
  				else ch1 = operande1.chaineLatexSansPar(varFor);
  			}
  			// else ch1 = operande1.chaineCalculSansPar(varFor); Correction dfaut version 4.7.2
  			else ch1 = operande1.chaineLatexSansPar(varFor);
  			if (operande2.nature() == natOperation) {
  				COperation oper2 = (COperation) operande2;
  				if (Operateur.estTest(oper2.operateur)) ch2 = operande2.chaineLatex(varFor);
  				else ch2 = operande2.chaineLatexSansPar(varFor);
  			}
  			else ch2 = operande2.chaineLatexSansPar(varFor);
  			
  			ch = ch1 + Operateur.chaineOperateurLatex(operateur) + ch2;
  	} // switch Operateur}
  	// Pour un calcul complexe nul la chane renvoye est "0+0*i" on renvoie alors "0"
  	if (ch.equals(zeroComp)) return zero;
  	// Au cas o le calcul commenait par un moins unaire, la chaine commence par "0-"
  	if (ch.substring(0, 2).equals("0-")) ch = ch.substring(1);
  	
  	return ch;		
	}
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		operande1.estDefiniParObjetDans(listeOb)
		&&
		operande2.estDefiniParObjetDans(listeOb);
	}	   
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    operateur = inps.readByte();
    operande1 = (CCalculBase) (inps.readCObject(list));
		operande2 = (CCalculBase) (inps.readCObject(list));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    operateur = inps.readByte();
    operande1 = (CCalculBase) (inps.readObject(list));
		operande2 = (CCalculBase) (inps.readObject(list));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(operateur);
    oups.writeObject(operande1);
    oups.writeObject(operande2);
  }
  
  @Override
	public boolean estConstant() {
		return operande1.estConstant() && operande2.estConstant();
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return operande1.deriveePossible(indiceVariable) && operande2.deriveePossible(indiceVariable);
  }

  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	return new COperation(listeProprietaire, operande1.calculAvecValeursRemplacees(),
  			operande2.calculAvecValeursRemplacees(), operateur) ;
  }
  
	// Ajout version 4.6
  // Modifi version 4.7. Renvoie toujours le membre de gauche mme en cas d'ingalit
  @Override
	public CCalculBase membreGauche() {
  	return operande1;
  	/*
  	if (Operateur.estTest(operateur))
  		return operande1;
  	else {
  		// Pour une ingalit, renvoie le membre contenant le terme suppos tre le plus grand
  		switch (operateur) {
	  		case Operateur.Sup :
	  		case Operateur.SupOuEgal :
	  			return operande1;
	  		case Operateur.Inf :
	  		case Operateur.InfOuEgal :
	  			return operande2;
	  		default :
	  			return null;
	  	}
  	}
  	*/
	}
	
	// Ajout version 4.6
	public CCalculBase membreDroit() {
  	return operande2;
    // Modifi version 4.7. Renvoie toujours le mebre de gauche mme en cas d'ingalit
  	/*
  	if (Operateur.estTest(operateur))
  		return operande2;
  	else {
  		// Pour une ingalit, renvoie le membre contenant le terme suppos tre le plus petit
  		switch (operateur) {
	  		case Operateur.Sup :
	  		case Operateur.SupOuEgal :
	  			return operande2;
	  		case Operateur.Inf :
	  		case Operateur.InfOuEgal :
	  			return operande1;
	  		default :
	  			return null;
	  	}
  	}
  	*/
	}
  
}
